﻿using System;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            HttpClient client = new HttpClient();

            string html = DownloadStringAsync(client, "https://www.daum.net").GetAwaiter().GetResult();

            Console.WriteLine(html);
        }

        #endregion

        #region 문자열 다운로드하기 (비동기) - DownloadStringAsync(httpClient, uri)

        /// <summary>
        /// 문자열 다운로드하기 (비동기)
        /// </summary>
        /// <param name="httpClient">HTTP 클라이언트</param>
        /// <param name="uri">URI</param>
        /// <returns>문자열 태스크</returns>
        private static async Task<string> DownloadStringAsync(HttpClient httpClient, string uri)
        {
            using CancellationTokenSource cancellationTokenSource = new CancellationTokenSource(TimeSpan.FromSeconds(3));

            Task<string> downloadTask = httpClient.GetStringAsync(uri);

            Task timeoutTask = Task.Delay(Timeout.InfiniteTimeSpan, cancellationTokenSource.Token);

            Task completedTask = await Task.WhenAny(downloadTask, timeoutTask);

            if(completedTask == timeoutTask)
            {
                return null;
            }

            return await downloadTask;
        }

        #endregion
    }
}